# =====================================================================
# xmltpl.awk: shell front-end to the envtoxml() AWK function.
#
# Copyright (c) 2002,2003 Carlo Strozzi
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
# =====================================================================
# $Id: xmlrpctpl.awk,v 1.4 2003/05/07 09:11:26 carlo Exp $

BEGIN {

  # Set default args before calling envtoxml(). I use the same
  # names here for clarity, but it is not necessary, of course,
  # as everything is localised by the function. Note that the
  # template root directory variable is the same as the one used for
  # standard HTML-based responses. To keep thing clean and manageable,
  # an application can either be HTML-based or XML-RPC based, but not
  # both. That variable will therefore contain different paths in the
  # cases. If an HTML application also needs to have an XML-RPC
  # interface, then two separate applications will have to be provided.
  # If the two need to share data files, that will have to be managed
  # through file-system links.

  if (tpl == "") tpl = "/dev/null"
  if (froot == "") froot = ENVIRON["CSA_ROOT"] "/xml/"
  if (trim != 1) trim = 0
  if (tago == "") tago = "$["
  if (tagc == "") tagc = "]"
  if (tago == tagc) exit(1)	# start- and end-tags must be different
  if (ofile == "") ofile = STDOUT

  usf = ENVIRON["CSA_ROOT"] "/var/user.d/" \
	ENVIRON["CSA_USER_PBC"] "/" ENVIRON["REMOTE_USER"]

  if ((rc = envtoxml(tpl,froot,eval,tbl,trim,tago,tagc,ofile))) exit(rc)

  # Test whether this is an authenticated user session
  # and save the current state in the USF if it is.

  if (ENVIRON["REMOTE_USER"] != "" && \
	ENVIRON["CSA_USER_ID"] != "" && \
	ENVIRON["CSA_USER_PBC"] ~ /^.\/.$/ && \
	ENVIRON["CSA_USF_CACHE"] != "") {

     # Create USF.
     if ((rc = creat(usf, O_TRUNC))) exit(rc)

     # Write selected status data to USF.
     for (env in ENVIRON) {
	 if (env ~ ENVIRON["CSA_USF_CACHE"])
	 	rcassign(env, ENVIRON[env], usf)
     }

     # Write new time-stamp to USF.
     rcassign("CSA_USF_TIME", ENVIRON["CSA_TIME_ISO"], usf)
  }
}

# EOF
