# =====================================================================
# AWK functions for CSA.
#
# Copyright (c) 2004 Carlo Strozzi
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License 
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
# =====================================================================
# $Id: _wwwarg.awk,v 1.1 2004/08/08 21:15:38 carlo Exp $
# =====================================================================
# string _nlsmap(string name, string value \
#			[, string program, string repl, int opt])
#
# Takes a name/value pair and looks for the corresponding NLS
# translation. If lookup fails returns the original value unchanged.
# If 'program' is not specified it defaults to the current program
# name. Occasionally, the same program may want to use multiple
# output forms, each one with its own mapping set. In those cases
# the "trick" is that the program calls this function passing
# the template name, rather than the program name, in the 'program'
# argument. Of course the local _NLSMAP will need to have been built
# accordingly. Furthermore, 'program' can be any string, even something
# like "dir/template", so this scheme turns out to be very flexible.
# The optional 'repl' parameter is a substitution token for those
# map entries that take a parameter in the form of a single '%s'.
# If the 'opt' flag is set to _O_CURR, then the result of the lookup
# will be returned in currency format, where applicable.
# =====================================================================

function _nlsmap(name,value,prog,repl,opt,		fmt) {

  # Now a null pgm value indicates a generic map lookup.

  #if (prog == _NULL) {
  #   prog = ENVIRON["CSA_PGM"]
  #   sub(/.*\001/,_NULL,prog)		# keep only leftmost value.
  #}

  if (_NLSMAP[prog,name,value] == _NULL) {

     if (opt == _O_CURR) {

	# Look-up the format string and make sure it is not empty,
	# or things may run wild.

	if ((fmt="%s" _nlsmap("currfmt")) == _NULL) fmt = "%.2f"
	return sprintf(fmt,_nlsmap("currency",_NULL,_NULL,repl),value)
     }

     return value
  }

  if (repl != _NULL) return sprintf(_NLSMAP[prog,name,value],repl)

  return _NLSMAP[prog,name,value]
}

