/* Written by and copyright Carlo Strozzi <carlos@linux.it>. */

/* cpmv: copy and then rename a file. This is a trivial all-in-one
   replacemente for the cp+mv command sequence. It is meant to
   spare processes in CSA commit operations.

   Copyright (C) 2001-2002 Carlo Strozzi <carlos@linux.it>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


   Usage:  cpmv from to mv

   Examples:

   $ cpmv file1 file2 file3

   This will copy file1 to file2 and then rename file2 into file3.

*/

static char Psz_RCS[] = "$Date: 2003/03/17 11:37:29 $";

#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>

#define PERMS 0644			/* This is hard-wired */

int main(int argc, char **argv) {

	int f1, f2, n;
	char buf[BUFSIZ];

	if (argc != 4) {
		fputs("Usage: cpmv from to mv\n", stderr);
		exit(1);
	}

	if ((f1 = open(argv[1], O_RDONLY)) < 0) {
		fprintf(stderr, "cpmv: can't open %s\n", argv[1]);
		exit(1);
	}

	if ((f2 = creat(argv[2], PERMS)) < 0) {
		fprintf(stderr, "cpmv: can't create %s\n", argv[2]);
		exit(1);
	}

	while ((n = read(f1, buf, BUFSIZ)) > 0)
		if (write(f2, buf, n) != n) {
			fputs("cpmv: write error", stderr);
			exit(1);
		}

	if (rename(argv[2], argv[3]) < 0) {
		fprintf(stderr, "cpmv: %s -> %s failed\n", argv[2], argv[3]);
		exit(1);
	}

	exit(0);

}

/* EOF */
