/* csa-unescape: copy stdin to stdout, applying the requested unescaping
   of special characters along the way. Of course this could be
   done just as well with a sed(1) script, but the latter would be
   much slower on large input, and could not cope easily with the
   transformation of newlines.

   Copyright (c) 2003,2004 Carlo Strozzi

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


   Usage:  csa-unescape [-z|-q]

   where:

     '-z' stands for 'binary zeroes'
     '-q' stands for 'binary zeroes', when input is quoted-printable

 ****************************************************************************/

static char Psz_RCS[] = "$Date: 2004/08/08 21:15:38 $";

#include <stdio.h>
#include <stdlib.h>

static void usage() {
  fprintf(stderr,"usage: csa-unescape [-z|-q]\n");
}

int main(int argc, char **argv) {

   int c, enc='z', escape=0;
   char buf[3];

   if ((argc > 1) &&
       (!strcmp(argv[1],"-h") || !strcmp(argv[1],"--help"))) {
      usage();
      exit(1);
   }

   if (argc > 1) {
      if (!strcmp(argv[1],"-z")) enc='z';
      else if (!strcmp(argv[1],"-q")) enc='q';
      else {
      	 usage();
      	 exit(1);
      }
   }

   if (enc == 'z') {

      /* mawk(1) un-escaping: unescape binary zeroes */

      while ((c = getchar()) != EOF) {

	    if (!escape && c == '\\') {
	       escape = 1;
	       continue;
	    }

	    if (escape && c == '0') c = '\0';

	    putchar(c);
	    escape = 0;
      }
   }

   else {			/* default is 'q' */

      /* mawk(1) un-escaping: unescape quoted-printable binary zeroes */

      while ((c = getchar()) != EOF) {

	    if (escape == 0 && c == '=') buf[escape++] = c;
	    else if (escape == 1) {
		 if (c == '0' || c == '3') buf[escape++] = c;
		 else {
		    buf[escape++] = '\0';
		    printf("%s", buf);
		    putchar(c);
		    escape = 0;
		 }
	    }
	    else if (escape == 2) {
		 if (c == '0' && buf[escape-1] == '0') putchar('\0');
		 else if ((c == 'D' || c == 'd') && buf[escape-1] == '3')
				putchar('=');
		 else {
		    buf[escape++] = '\0';
		    printf("%s", buf);
		    putchar(c);
		 }
		 escape = 0;
	    }
	    else putchar(c);
      }
   }


   exit(0);
}

/* End of program */
